#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/xattr.h>
#include <linux/version.h>
#include <linux/user_namespace.h>
#include <linux/lsm_hooks.h>
#include <linux/security_ops.h>

#define XATTR_SECURITY_PREFIX "security."

#define XATTR_BOX_SUFFIX "boxadm"
#define XATTR_NAME_BOX (XATTR_SECURITY_PREFIX XATTR_BOX_SUFFIX)

#define XATTR_EXECTL_SUFFIX "exectl"
#define XATTR_NAME_EXECTL (XATTR_SECURITY_PREFIX XATTR_EXECTL_SUFFIX)


static int ch_inode_listxattr(struct dentry *dentry)
{
	if (strstr(current->comm, "listxattr")) {
		//__vfs_removexattr_noperm(dentry, XATTR_NAME_BOX);
		//__vfs_removexattr_noperm(dentry, XATTR_NAME_EXECTL);
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0)
		__vfs_removexattr(dentry, XATTR_NAME_BOX);
		__vfs_removexattr(dentry, XATTR_NAME_EXECTL);
#else
		__vfs_removexattr(&init_user_ns, dentry, XATTR_NAME_BOX);
		__vfs_removexattr(&init_user_ns, dentry, XATTR_NAME_EXECTL);
#endif
	}
	return 0;
}


static struct security_operations chlistxattr_ops = {
	.name = "chlistxattr",
	.inode_listxattr = ch_inode_listxattr,
};

static int __init chlistxattr_init(void)
{
	int ret;

	ret = security_add_external_ops(&chlistxattr_ops);
	if (ret < 0)
		return ret;

	return 0;
}

static void __exit chlistxattr_exit(void)
{
	security_del_external_ops(&chlistxattr_ops);
}

module_init(chlistxattr_init);
module_exit(chlistxattr_exit);

MODULE_VERSION("2.1");
MODULE_LICENSE("GPL");
MODULE_AUTHOR("huanfuxi");
